@@ -17,13 +17,13 @@
 %                               November 2001                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -173,7 +173,7 @@ typedef struct
     green,
     blue,
     alpha;
-} XCFPixelPacket;
+} XCFPixelInfo;
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -282,7 +282,8 @@ static CompositeOperator GIMPBlendModeToCompositeOperator(
 %
 %  The format of the ReadBlobStringWithLongSize method is:
 %
-%      char *ReadBlobStringWithLongSize(Image *image,char *string)
+%      char *ReadBlobStringWithLongSize(Image *image,char *string,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -290,9 +291,12 @@ static CompositeOperator GIMPBlendModeToCompositeOperator(
 %
 %    o string: the address of a character buffer.
 %
+%    o exception: return any errors or warnings in this structure.
+%
 */
 
-static char *ReadBlobStringWithLongSize(Image *image,char *string,size_t max)
+static char *ReadBlobStringWithLongSize(Image *image,char *string,size_t max,
+  ExceptionInfo *exception)
 {
   int
     c;
@@ -322,17 +326,15 @@ static char *ReadBlobStringWithLongSize(Image *image,char *string,size_t max)
   string[i]='\0';
   offset=SeekBlob(image,(MagickOffsetType) (length-i),SEEK_CUR);
   if (offset < 0)
-    (void) ThrowMagickException(&image->exception,GetMagickModule(),
+    (void) ThrowMagickException(exception,GetMagickModule(),
       CorruptImageError,"ImproperImageHeader","`%s'",image->filename);
   return(string);
 }
 
 static MagickBooleanType load_tile(Image *image,Image *tile_image,
-  XCFDocInfo *inDocInfo,XCFLayerInfo *inLayerInfo,size_t data_length)
+  XCFDocInfo *inDocInfo,XCFLayerInfo *inLayerInfo,size_t data_length,
+  ExceptionInfo *exception)
 {
-  ExceptionInfo
-    *exception;
-
   ssize_t
     y;
 
@@ -351,7 +353,7 @@ static MagickBooleanType load_tile(Image *image,Image *tile_image,
   unsigned char
     *graydata;
 
-  XCFPixelPacket
+  XCFPixelInfo
     *xcfdata,
     *xcfodata;
 
@@ -364,9 +366,9 @@ static MagickBooleanType load_tile(Image *image,Image *tile_image,
   if (extent > data_length)
     ThrowBinaryException(CorruptImageError,"NotEnoughPixelData",
       image->filename);
-  xcfdata=(XCFPixelPacket *) AcquireQuantumMemory(MagickMax(data_length,
+  xcfdata=(XCFPixelInfo *) AcquireQuantumMemory(MagickMax(data_length,
     tile_image->columns*tile_image->rows),sizeof(*xcfdata));
-  if (xcfdata == (XCFPixelPacket *) NULL)
+  if (xcfdata == (XCFPixelInfo *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
   xcfodata=xcfdata;
@@ -374,11 +376,10 @@ static MagickBooleanType load_tile(Image *image,Image *tile_image,
   count=ReadBlob(image,data_length,(unsigned char *) xcfdata);
   if (count != (ssize_t) data_length)
     {
-      xcfodata=(XCFPixelPacket *) RelinquishMagickMemory(xcfodata);
+      xcfodata=(XCFPixelInfo *) RelinquishMagickMemory(xcfodata);
       ThrowBinaryException(CorruptImageError,"NotEnoughPixelData",
         image->filename);
     }
-  exception=(&image->exception);
   for (y=0; y < (ssize_t) tile_image->rows; y++)
   {
     q=GetAuthenticPixels(tile_image,0,y,tile_image->columns,1,exception);
@@ -414,21 +415,20 @@ static MagickBooleanType load_tile(Image *image,Image *tile_image,
      if (SyncAuthenticPixels(tile_image,exception) == MagickFalse)
        break;
   }
-  xcfodata=(XCFPixelPacket *) RelinquishMagickMemory(xcfodata);
+  xcfodata=(XCFPixelInfo *) RelinquishMagickMemory(xcfodata);
   return MagickTrue;
 }
 
 static MagickBooleanType load_tile_rle(Image *image,Image *tile_image,
-  XCFDocInfo *inDocInfo,XCFLayerInfo *inLayerInfo,size_t data_length)
+  XCFDocInfo *inDocInfo,XCFLayerInfo *inLayerInfo,size_t data_length,
+  ExceptionInfo *exception)
 {
-  ExceptionInfo
-    *exception;
-
   MagickOffsetType
     size;
 
   Quantum
-    alpha;
+    alpha,
+    data;
 
   register PixelPacket
     *q;
@@ -443,7 +443,6 @@ static MagickBooleanType load_tile_rle(Image *image,Image *tile_image,
     j;
 
   unsigned char
-    data,
     pixel,
     *xcfdata,
     *xcfodata,
@@ -457,9 +456,8 @@ static MagickBooleanType load_tile_rle(Image *image,Image *tile_image,
   xcfodata=xcfdata;
   count=ReadBlob(image, (size_t) data_length, xcfdata);
   xcfdatalimit = xcfodata+count-1;
-  exception=(&image->exception);
   alpha=ScaleCharToQuantum((unsigned char) inLayerInfo->alpha);
-  for (i=0; i < (ssize_t) bytes_per_pixel; i++)
+  for (i=0; i < bytes_per_pixel; i++)
   {
     q=GetAuthenticPixels(tile_image,0,0,tile_image->columns,tile_image->rows,
       exception);
@@ -489,16 +487,16 @@ static MagickBooleanType load_tile_rle(Image *image,Image *tile_image,
             goto bogus_rle;
           while (length-- > 0)
           {
-            data=(*xcfdata++);
+            data=ScaleCharToQuantum(*xcfdata++);
             switch (i)
             {
               case 0:
               {
-                SetPixelRed(q,ScaleCharToQuantum(data));
+                SetPixelRed(q,data);
                 if (inDocInfo->image_type == GIMP_GRAY)
                   {
-                    SetPixelGreen(q,ScaleCharToQuantum(data));
-                    SetPixelBlue(q,ScaleCharToQuantum(data));
+                    SetPixelGreen(q,data);
+                    SetPixelBlue(q,data);
                   }
                 else
                   {
@@ -511,19 +509,19 @@ static MagickBooleanType load_tile_rle(Image *image,Image *tile_image,
               case 1:
               {
                 if (inDocInfo->image_type == GIMP_GRAY)
-                  SetPixelAlpha(q,ScaleCharToQuantum(data));
+                  SetPixelAlpha(q,data);
                 else
-                  SetPixelGreen(q,ScaleCharToQuantum(data));
+                  SetPixelGreen(q,data);
                 break;
               }
               case 2:
               {
-                SetPixelBlue(q,ScaleCharToQuantum(data));
+                SetPixelBlue(q,data);
                 break;
               }
               case 3:
               {
-                SetPixelAlpha(q,ScaleCharToQuantum(data));
+                SetPixelAlpha(q,data);
                 break;
               }
             }
@@ -548,16 +546,16 @@ static MagickBooleanType load_tile_rle(Image *image,Image *tile_image,
           pixel=(*xcfdata++);
           for (j=0; j < (ssize_t) length; j++)
           {
-            data=pixel;
+            data=ScaleCharToQuantum(pixel);
             switch (i)
             {
               case 0:
               {
-                SetPixelRed(q,ScaleCharToQuantum(data));
+                SetPixelRed(q,data);
                 if (inDocInfo->image_type == GIMP_GRAY)
                   {
-                    SetPixelGreen(q,ScaleCharToQuantum(data));
-                    SetPixelBlue(q,ScaleCharToQuantum(data));
+                    SetPixelGreen(q,data);
+                    SetPixelBlue(q,data);
                   }
                 else
                   {
@@ -570,19 +568,19 @@ static MagickBooleanType load_tile_rle(Image *image,Image *tile_image,
               case 1:
               {
                 if (inDocInfo->image_type == GIMP_GRAY)
-                  SetPixelAlpha(q,ScaleCharToQuantum(data));
+                  SetPixelAlpha(q,data);
                 else
-                  SetPixelGreen(q,ScaleCharToQuantum(data));
+                  SetPixelGreen(q,data);
                 break;
               }
               case 2:
               {
-                SetPixelBlue(q,ScaleCharToQuantum(data));
+                SetPixelBlue(q,data);
                 break;
               }
               case 3:
               {
-                SetPixelAlpha(q,ScaleCharToQuantum(data));
+                SetPixelAlpha(q,data);
                 break;
               }
             }
@@ -597,17 +595,14 @@ static MagickBooleanType load_tile_rle(Image *image,Image *tile_image,
   return(MagickTrue);
 
   bogus_rle:
-    if (xcfodata != (unsigned char *) NULL)
-      xcfodata=(unsigned char *) RelinquishMagickMemory(xcfodata);
+  if (xcfodata != (unsigned char *) NULL)
+    xcfodata=(unsigned char *) RelinquishMagickMemory(xcfodata);
   return(MagickFalse);
 }
 
 static MagickBooleanType load_level(Image *image,XCFDocInfo *inDocInfo,
-  XCFLayerInfo *inLayerInfo)
+  XCFLayerInfo *inLayerInfo,ExceptionInfo *exception)
 {
-  ExceptionInfo
-    *exception;
-
   int
     destLeft = 0,
     destTop = 0;
@@ -636,7 +631,6 @@ static MagickBooleanType load_level(Image *image,XCFDocInfo *inDocInfo,
     tile_image_height;
 
   /* start reading the data */
-  exception=inDocInfo->exception;
   width=ReadBlobMSBLong(image);
   height=ReadBlobMSBLong(image);
 
@@ -645,6 +639,9 @@ static MagickBooleanType load_level(Image *image,XCFDocInfo *inDocInfo,
     and we can simply return.
   */
   offset=(MagickOffsetType) ReadBlobMSBLong(image);
+  if (EOFBlob(image) != MagickFalse)
+    ThrowBinaryException(CorruptImageError,"UnexpectedEndOfFile", 
+      image->filename);
   if (offset == 0)
     return(MagickTrue);
   /*
@@ -658,14 +655,14 @@ static MagickBooleanType load_level(Image *image,XCFDocInfo *inDocInfo,
     status=MagickFalse;
     if (offset == 0)
       ThrowBinaryException(CorruptImageError,"NotEnoughTiles",image->filename);
-    /* save the current position as it is where the
-     *  next tile offset is stored.
-     */
+    /*
+      Save the current position as it is where the next tile offset is stored.
+    */
     saved_pos=TellBlob(image);
     /* read in the offset of the next tile so we can calculate the amount
        of data needed for this tile*/
-    offset2=(MagickOffsetType)ReadBlobMSBLong(image);
-    if ((MagickSizeType) offset2 > GetBlobSize(image))
+    offset2=(MagickOffsetType) ReadBlobMSBLong(image);
+    if ((MagickSizeType) offset2 >= inDocInfo->file_size)
       ThrowBinaryException(CorruptImageError,"InsufficientImageDataInFile",
         image->filename);
     /* if the offset is 0 then we need to read in the maximum possible
@@ -673,12 +670,13 @@ static MagickBooleanType load_level(Image *image,XCFDocInfo *inDocInfo,
     if (offset2 == 0)
       offset2=(MagickOffsetType) (offset + TILE_WIDTH * TILE_WIDTH * 4* 1.5);
     /* seek to the tile offset */
-    if (SeekBlob(image, offset, SEEK_SET) != offset)
+    if ((offset > offset2) || ( SeekBlob(image, offset, SEEK_SET) != offset))
       ThrowBinaryException(CorruptImageError,"InsufficientImageDataInFile",
         image->filename);
 
-      /* allocate the image for the tile
-        NOTE: the last tile in a row or column may not be a full tile!
+      /*
+        Allocate the image for the tile.  NOTE: the last tile in a row or
+        column may not be a full tile!
       */
       tile_image_width=(size_t) (destLeft == (int) ntile_cols-1 ?
         (int) width % TILE_WIDTH : TILE_WIDTH);
@@ -699,13 +697,12 @@ static MagickBooleanType load_level(Image *image,XCFDocInfo *inDocInfo,
       switch (inDocInfo->compression)
       {
         case COMPRESS_NONE:
-          if (load_tile(image,tile_image,inDocInfo,inLayerInfo,(size_t) (offset2-offset)) == 0)
-            status=MagickTrue;
+          status=load_tile(image,tile_image,inDocInfo,inLayerInfo,(size_t)
+            (offset2-offset),&image->exception);
           break;
         case COMPRESS_RLE:
-          if (load_tile_rle (image,tile_image,inDocInfo,inLayerInfo,
-              (int) (offset2-offset)) == 0)
-            status=MagickTrue;
+          status=load_tile_rle(image,tile_image,inDocInfo,inLayerInfo,(size_t)
+            (offset2-offset),&image->exception);
           break;
         case COMPRESS_ZLIB:
           tile_image=DestroyImage(tile_image);
@@ -718,10 +715,13 @@ static MagickBooleanType load_level(Image *image,XCFDocInfo *inDocInfo,
       }
 
       /* composite the tile onto the layer's image, and then destroy it */
-      (void) CompositeImage(inLayerInfo->image,CopyCompositeOp,tile_image,
-        destLeft * TILE_WIDTH,destTop*TILE_HEIGHT);
+      if (status != MagickFalse)
+        (void) CompositeImage(inLayerInfo->image,CopyCompositeOp,tile_image,
+          destLeft * TILE_WIDTH,destTop*TILE_HEIGHT);
       tile_image=DestroyImage(tile_image);
 
+      if (status == MagickFalse)
+        return(MagickFalse);
       /* adjust tile position */
       destLeft++;
       if (destLeft >= (int) ntile_cols)
@@ -729,8 +729,6 @@ static MagickBooleanType load_level(Image *image,XCFDocInfo *inDocInfo,
           destLeft = 0;
           destTop++;
         }
-      if (status != MagickFalse)
-        return(MagickFalse);
       /* restore the saved position so we'll be ready to
        *  read the next offset.
        */
@@ -751,23 +749,18 @@ static MagickBooleanType load_hierarchy(Image *image,XCFDocInfo *inDocInfo,
     offset,
     junk;
 
-  size_t
-    width,
-    height,
-    bytes_per_pixel;
-
-  width=ReadBlobMSBLong(image);
-  (void) width;
-  height=ReadBlobMSBLong(image);
-  (void) height;
-  bytes_per_pixel=inDocInfo->bytes_per_pixel=ReadBlobMSBLong(image);
-  (void) bytes_per_pixel;
+  (void) ReadBlobMSBLong(image);  /* width */
+  (void) ReadBlobMSBLong(image);  /* height */
+  inDocInfo->bytes_per_pixel=ReadBlobMSBLong(image);
 
   /* load in the levels...we make sure that the number of levels
    *  calculated when the TileManager was created is the same
    *  as the number of levels found in the file.
    */
   offset=(MagickOffsetType) ReadBlobMSBLong(image);  /* top level */
+  if ((MagickSizeType) offset >= GetBlobSize(image))
+    ThrowBinaryImageException(CorruptImageError,"InsufficientImageDataInFile",
+      image->filename);
 
   /* discard offsets for layers below first, if any.
    */
@@ -784,11 +777,11 @@ static MagickBooleanType load_hierarchy(Image *image,XCFDocInfo *inDocInfo,
 
   /* seek to the level offset */
   if (SeekBlob(image, offset, SEEK_SET) != offset)
-    ThrowBinaryException(CorruptImageError,"InsufficientImageDataInFile",
+    ThrowBinaryImageException(CorruptImageError,"InsufficientImageDataInFile",
       image->filename);
 
   /* read in the level */
-  if (load_level (image, inDocInfo, inLayer) == 0)
+  if (load_level (image, inDocInfo, inLayer, &image->exception) == 0)
     return(MagickFalse);
   /* restore the saved position so we'll be ready to
    *  read the next offset.
@@ -829,13 +822,12 @@ static MagickBooleanType ReadOneLayer(const ImageInfo *image_info,Image* image,
   outLayer->height = ReadBlobMSBLong(image);
   outLayer->type = ReadBlobMSBLong(image);
   (void) ReadBlobStringWithLongSize(image, outLayer->name,
-    sizeof(outLayer->name));
+    sizeof(outLayer->name),&image->exception);
   if (EOFBlob(image) != MagickFalse)
-    ThrowBinaryException(CorruptImageError,"InsufficientImageDataInFile",
+    ThrowBinaryImageException(CorruptImageError,"InsufficientImageDataInFile",
       image->filename);
-  if ((outLayer->width < 1) || (outLayer->width > image->columns) ||
-      (outLayer->height < 1) || (outLayer->height > image->rows))
-    ThrowBinaryException(CorruptImageError,"ImproperImageHeader",
+  if ((outLayer->width == 0) || (outLayer->height == 0))
+    ThrowBinaryImageException(CorruptImageError,"ImproperImageHeader",
       image->filename);
   /* read the layer properties! */
   foundPropEnd = 0;
@@ -918,7 +910,7 @@ static MagickBooleanType ReadOneLayer(const ImageInfo *image_info,Image* image,
         amount = (ssize_t) MagickMin(16, prop_size);
         amount = ReadBlob(image, (size_t) amount, (unsigned char *) &buf);
         if (!amount)
-          ThrowBinaryException(CorruptImageError,"CorruptImage",
+          ThrowBinaryImageException(CorruptImageError,"CorruptImage",
             image->filename);
         prop_size -= (size_t) MagickMin(16, (size_t) amount);
         }
@@ -926,7 +918,9 @@ static MagickBooleanType ReadOneLayer(const ImageInfo *image_info,Image* image,
       break;
     }
   }
-
+  if (EOFBlob(image) != MagickFalse)
+    ThrowBinaryImageException(CorruptImageError,"UnexpectedEndOfFile",
+      image->filename);
   if (foundPropEnd == MagickFalse)
     return(MagickFalse);
   /* allocate the image for this layer */
@@ -952,6 +946,8 @@ static MagickBooleanType ReadOneLayer(const ImageInfo *image_info,Image* image,
     return(MagickFalse);
   status=SetImageExtent(outLayer->image,outLayer->image->columns,
     outLayer->image->rows);
+  if (status != MagickFalse)
+    status=ResetImagePixels(image,&image->exception);
   if (status == MagickFalse)
     {
       InheritException(&image->exception,&outLayer->image->exception);
@@ -961,7 +957,8 @@ static MagickBooleanType ReadOneLayer(const ImageInfo *image_info,Image* image,
   /* clear the image based on the layer opacity */
   outLayer->image->background_color.opacity=
     ScaleCharToQuantum((unsigned char) (255-outLayer->alpha));
-  (void) SetImageBackgroundColor(outLayer->image);
+  if (outLayer->alpha != 255U)
+    outLayer->image->matte=MagickTrue;
 
   InitXCFImage(outLayer);
 
@@ -980,8 +977,8 @@ static MagickBooleanType ReadOneLayer(const ImageInfo *image_info,Image* image,
   /* read in the hierarchy */
   offset=SeekBlob(image, (MagickOffsetType) hierarchy_offset, SEEK_SET);
   if (offset != (MagickOffsetType) hierarchy_offset)
-    (void) ThrowMagickException(&image->exception,GetMagickModule(),
-      CorruptImageError,"InvalidImageHeader","`%s'",image->filename);
+    ThrowBinaryImageException(CorruptImageError,"InvalidImageHeader",
+      image->filename);
   if (load_hierarchy (image, inDocInfo, outLayer) == 0)
     return(MagickFalse);
 
@@ -1120,14 +1117,16 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       InheritException(exception,&image->exception);
       return(DestroyImageList(image));
     }
+  if (status != MagickFalse)
+    status=ResetImagePixels(image,exception);
+  if (image_type == GIMP_INDEXED)
+    ThrowReaderException(CoderError,"ColormapTypeNotSupported");
   if (image_type == GIMP_RGB)
     SetImageColorspace(image,sRGBColorspace);
-  else
-    if (image_type == GIMP_GRAY)
-      SetImageColorspace(image,GRAYColorspace);
-    else
-      if (image_type == GIMP_INDEXED)
-        ThrowReaderException(CoderError,"ColormapTypeNotSupported");
+  else if (image_type == GIMP_GRAY)
+    SetImageColorspace(image,GRAYColorspace);
+   else
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   (void) SetImageBackgroundColor(image);
   (void) SetImageOpacity(image,OpaqueOpacity); 
   /*
@@ -1285,7 +1284,7 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /* size_t digits =  */ (void) ReadBlobMSBLong(image);
         for (i=0; i<5; i++)
          (void) ReadBlobStringWithLongSize(image, unit_string,
-           sizeof(unit_string));
+           sizeof(unit_string),&image->exception);
       }
      break;
 
@@ -1431,7 +1430,7 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /* now reverse the order of the layers as they are put
            into subimages
         */
-        for (j=(long) number_layers-1; j >= 0; j--)
+        for (j=(ssize_t) number_layers-1; j >= 0; j--)
           AppendImageToList(&image,layer_info[j].image);
       }
 #endif
