@@ -195,57 +195,7 @@ MagickExport void AnnotateComponentTerminus(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  AnnotateImage() annotates an image with text.  Optionally you can include
-%  any of the following bits of information about the image by embedding
-%  the appropriate special characters:
-%
-%    \n   newline
-%    \r   carriage return
-%    <    less-than character.
-%    >    greater-than character.
-%    &    ampersand character.
-%    %%   a percent sign
-%    %b   file size of image read in
-%    %c   comment meta-data property
-%    %d   directory component of path
-%    %e   filename extension or suffix
-%    %f   filename (including suffix)
-%    %g   layer canvas page geometry   (equivalent to "%Wx%H%X%Y")
-%    %h   current image height in pixels
-%    %i   image filename (note: becomes output filename for "info:")
-%    %k   CALCULATED: number of unique colors
-%    %l   label meta-data property
-%    %m   image file format (file magic)
-%    %n   number of images in current image sequence
-%    %o   output filename  (used for delegates)
-%    %p   index of image in current image list
-%    %q   quantum depth (compile-time constant)
-%    %r   image class and colorspace
-%    %s   scene number (from input unless re-assigned)
-%    %t   filename without directory or extension (suffix)
-%    %u   unique temporary filename (used for delegates)
-%    %w   current width in pixels
-%    %x   x resolution (density)
-%    %y   y resolution (density)
-%    %z   image depth (as read in unless modified, image save depth)
-%    %A   image transparency channel enabled (true/false)
-%    %C   image compression type
-%    %D   image GIF dispose method
-%    %G   original image size (%wx%h; before any resizes)
-%    %H   page (canvas) height
-%    %M   Magick filename (original file exactly as given,  including read mods)
-%    %O   page (canvas) offset ( = %X%Y )
-%    %P   page (canvas) size ( = %Wx%H )
-%    %Q   image compression quality ( 0 = default )
-%    %S   ?? scenes ??
-%    %T   image time delay (in centi-seconds)
-%    %U   image resolution units
-%    %W   page (canvas) width
-%    %X   page (canvas) x offset (including sign)
-%    %Y   page (canvas) y offset (including sign)
-%    %Z   unique filename (used for delegates)
-%    %@   CALCULATED: trim bounding box (without actually trimming)
-%    %#   CALCULATED: 'signature' hash of image values
+%  AnnotateImage() annotates an image with text.
 %
 %  The format of the AnnotateImage method is:
 %
@@ -262,7 +212,9 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
   const DrawInfo *draw_info)
 {
   char
+    *p,
     primitive[MaxTextExtent],
+    *text,
     **textlist;
 
   DrawInfo
@@ -284,9 +236,6 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
   register ssize_t
     i;
 
-  size_t
-    length;
-
   TypeMetric
     metrics;
 
@@ -304,16 +253,41 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
     return(MagickFalse);
   if (*draw_info->text == '\0')
     return(MagickTrue);
-  textlist=StringToList(draw_info->text);
-  if (textlist == (char **) NULL)
-    return(MagickFalse);
-  length=strlen(textlist[0]);
-  for (i=1; textlist[i] != (char *) NULL; i++)
-    if (strlen(textlist[i]) > length)
-      length=strlen(textlist[i]);
-  number_lines=(size_t) i;
   annotate=CloneDrawInfo((ImageInfo *) NULL,draw_info);
+  text=annotate->text;
+  annotate->text=(char *) NULL;
   annotate_info=CloneDrawInfo((ImageInfo *) NULL,draw_info);
+  number_lines=1;
+  for (p=text; *p != '\0'; p++)
+    if (*p == '\n')
+      number_lines++;
+  textlist=AcquireQuantumMemory(number_lines+1,sizeof(*textlist));
+  if (textlist == (char **) NULL)
+    {
+      annotate_info=DestroyDrawInfo(annotate_info);
+      annotate=DestroyDrawInfo(annotate);
+      text=DestroyString(text);
+      return(MagickFalse);
+    }
+  p=text;
+  for (i=0; i < number_lines; i++)
+  {
+    char
+      *q;
+
+    textlist[i]=p;
+    for (q=p; *q != '\0'; q++)
+      if ((*q == '\r') || (*q == '\n'))
+        break;
+    if (*q == '\r')
+      {
+        *q='\0';
+        q++;
+      }
+    *q='\0';
+    p=q+1;
+  }
+  textlist[i]=(char *) NULL;
   SetGeometry(image,&geometry);
   SetGeometryInfo(&geometry_info);
   if (annotate_info->geometry != (char *) NULL)
@@ -323,19 +297,29 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
       (void) ParseGeometry(annotate_info->geometry,&geometry_info);
     }
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
-    return(MagickFalse);
+    {
+      annotate_info=DestroyDrawInfo(annotate_info);
+      annotate=DestroyDrawInfo(annotate);
+      textlist=(char **) RelinquishMagickMemory(textlist);
+      text=DestroyString(text);
+      return(MagickFalse);
+    }
   if (IsGrayColorspace(image->colorspace) != MagickFalse)
     (void) SetImageColorspace(image,sRGBColorspace);
   status=MagickTrue;
+  (void) memset(&metrics,0,sizeof(metrics));
   for (i=0; textlist[i] != (char *) NULL; i++)
   {
+    if (*textlist[i] == '\0')
+      continue;
     /*
       Position text relative to image.
     */
     annotate_info->affine.tx=geometry_info.xi-image->page.x;
     annotate_info->affine.ty=geometry_info.psi-image->page.y;
     (void) CloneString(&annotate->text,textlist[i]);
-    (void) GetTypeMetrics(image,annotate,&metrics);
+    if ((metrics.width == 0) || (annotate->gravity != NorthWestGravity))
+      (void) GetTypeMetrics(image,annotate,&metrics);
     height=(ssize_t) (metrics.ascent-metrics.descent+
       draw_info->interline_spacing+0.5);
     switch (annotate->gravity)
@@ -534,9 +518,8 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
   */
   annotate_info=DestroyDrawInfo(annotate_info);
   annotate=DestroyDrawInfo(annotate);
-  for (i=0; textlist[i] != (char *) NULL; i++)
-    textlist[i]=DestroyString(textlist[i]);
   textlist=(char **) RelinquishMagickMemory(textlist);
+  text=DestroyString(text);
   return(status);
 }
 
